Attribute VB_Name = "mAPIDefs"
Option Explicit

' Brought to you by:
'   Brad Martinez
'   btmtz@msn.com
'   btmtz@aol.com
'   http://members.aol.com/btmtz/vb

' Retrieves a pointer to the shell's IMalloc interface.
' Returns NOERROR if successful or or E_FAIL otherwise.
Declare Function SHGetMalloc Lib "Shell32" (ppMalloc As Long) As Long

' Retrieves a pointer to the desktop folder's IShellFolder interface.
' Returns NOERROR if successful or an OLE-defined error result otherwise.
Declare Function SHGetDesktopFolder Lib "Shell32" (ppshf As Long) As Long

' Retrieves the location of a special (system) folder.
' Returns NOERROR if successful or an OLE-defined error result otherwise.
Declare Function SHGetSpecialFolderLocation Lib "shell32.dll" _
                              (ByVal hwndOwner As Long, _
                              ByVal nFolder As Long, _
                              pidl As Long) As Long
' nFolder values
Public Const CSIDL_DESKTOP = &H0                      ' Desktop virtual folder
Public Const CSIDL_PROGRAMS = &H2                  ' Programs folder
Public Const CSIDL_CONTROLS = &H3                    ' Control Panel virtual folder
Public Const CSIDL_PRINTERS = &H4                      ' Printers virtual folder
Public Const CSIDL_PERSONAL = &H5                    ' Documents folder
Public Const CSIDL_FAVORITES = &H6                    ' Internet Explorer URLs
Public Const CSIDL_STARTUP = &H7                       ' Startup folder
Public Const CSIDL_RECENT = &H8                         ' Recent folder
Public Const CSIDL_SENDTO = &H9                         ' SendTo folder
Public Const CSIDL_BITBUCKET = &HA                    ' Recycle bin virtual folder
Public Const CSIDL_STARTMENU = &HB                  ' Start Menu folder
Public Const CSIDL_DESKTOPDIRECTORY = &H10   ' Win\Desktop folder
Public Const CSIDL_DRIVES = &H11                         ' My Computer virtual folder
Public Const CSIDL_NETWORK = &H12                    ' Network Neighborhood, virtual folder
Public Const CSIDL_NETHOOD = &H13                     ' Win\Nethood folder
Public Const CSIDL_FONTS = &H14                          ' Fonts virtual folder
Public Const CSIDL_TEMPLATES = &H15                 ' ShellNew folder

' SH function's successful rtn val (not used here)
'Public Const NOERROR = 0

' Moves a block of memory from one location to another, may overlap.
Declare Sub MoveMemory Lib "kernel32" Alias "RtlMoveMemory" _
                      (pDest As Any, _
                      pSource As Any, _
                      ByVal ByteLen As Long)

' Converts a ANSII str to a Unicode str.
' Specify -1 for cchWideChar and 0 for cchMultiByte to rtn str len.
Declare Function MultiByteToWideChar Lib "kernel32" _
                            (ByVal CodePage As Long, _
                            ByVal dwFlags As Long, _
                            lpMultiByteStr As Any, _
                            ByVal cchMultiByte As Long, _
                            lpWideCharStr As Any, _
                            ByVal cchWideChar As Long) As Long

' Converts a Unicode str to a ANSII str.
' Specify -1 for cchWideChar and 0 for cchMultiByte to rtn str len.
Declare Function WideCharToMultiByte Lib "kernel32" _
                            (ByVal CodePage As Long, _
                            ByVal dwFlags As Long, _
                            lpWideCharStr As Any, _
                            ByVal cchWideChar As Long, _
                            lpMultiByteStr As Any, _
                            ByVal cchMultiByte As Long, _
                            ByVal lpDefaultChar As String, _
                            ByVal lpUsedDefaultChar As Long) As Long
' CodePage
Public Const CP_OEMCP = 1   ' OEM code page

' Determines whether a buffer *probably* contains a form of Unicode text.
Declare Function IsTextUnicode Lib "advapi32" _
                            (lpBuffer As Any, _
                            ByVal cb As Long, _
                            lpi As Long) As Long

Public Const IS_TEXT_UNICODE_UNICODE_MASK = &HF   ' lpi

Declare Function FormatMessage Lib "kernel32" Alias "FormatMessageA" _
                            (ByVal dwFlags As Long, _
                            lpSource As Any, _
                            ByVal dwMessageId As Long, _
                            ByVal dwLanguageId As Long, _
                            ByVal lpBuffer As String, _
                            ByVal nSize As Long, _
                            Arguments As Long) As Long
' dwFlags
Public Const FORMAT_MESSAGE_FROM_SYSTEM = &H1000
Public Const FORMAT_MESSAGE_IGNORE_INSERTS = &H200
Public Const FORMAT_MESSAGE_MAX_WIDTH_MASK = &HFF
' dwLanguageId
Public Const LANG_USER_DEFAULT = &H400&
'

Public Function SUCCEEDED(dwIn As Long) As Boolean
'  SUCCEEDED = (dwIn >= 0)
  If (dwIn >= 0) Then
    SUCCEEDED = True
  Else
    Dim sErrDesc As String * 256
    FormatMessage FORMAT_MESSAGE_FROM_SYSTEM Or _
                            FORMAT_MESSAGE_IGNORE_INSERTS Or _
                            FORMAT_MESSAGE_MAX_WIDTH_MASK, _
                            ByVal 0&, dwIn, LANG_USER_DEFAULT, _
                            ByVal sErrDesc, 256&, 0&
    Debug.Print "APIErr &H" & Hex(dwIn) & ": " & GetStrFromBuffer(sErrDesc)
  End If
End Function

' Returns the string before first null char encountered (if any) from either
' a null terminated Unicode or ANSII string pointer.
' lpsz = memory address of first byte in string.
' fIsUnicodeStr = flag indicating whether lpsz points to a Unicode or ANSII str.
' The calling proc *must* be certain what type of str is pointed to by lpsz.
' The IsUnicodeStr function isn't used here as it can be unreliable.

Public Function GetStrFromPtr(lpsz As Long, _
                                              fIsUnicodeStr As Boolean) As String
  Dim nBytes As Integer
  If fIsUnicodeStr Then
    nBytes = GetStrLenFromPtrW(lpsz)
  Else   ' ANSII
    nBytes = GetStrLenFromPtrA(lpsz)
  End If
  
  ReDim ab(nBytes) As Byte   ' zero-based (nBytes + 1 elements)
  MoveMemory ab(0), ByVal lpsz, nBytes
  ' StrConv used for both ANSII and Unicode strings
  GetStrFromPtr = GetStrFromBuffer(StrConv(ab(), vbUnicode))

End Function

' Rtns the byte length of the Unicode str (2 bytes/char) pointed to by lpszW.

Public Function GetStrLenFromPtrW(lpszW As Long) As Long
  GetStrLenFromPtrW = WideCharToMultiByte(CP_OEMCP, 0, _
                                                                    ByVal lpszW, -1, 0, _
                                                                    0, vbNullString, 0) * 2
End Function

' Rtns the byte length of the ANSII str (1 byte/char) pointed to by lpszA.

Public Function GetStrLenFromPtrA(lpszA As Long) As Long
  GetStrLenFromPtrA = MultiByteToWideChar(CP_OEMCP, 0, _
                                                                    ByVal lpszA, -1, 0, 0)
End Function

' Returns the string before first null char encountered (if any) from either
' a null terminated Unicode or ANSII string.
' The IsUnicodeStr function is used here as it is assumed that sz is
' terminated only by null char(s).

Public Function GetStrFromBuffer(sz As String) As String
  If IsUnicodeStr(sz) Then sz = StrConv(sz, vbFromUnicode)
  If InStr(sz, vbNullChar) Then
    GetStrFromBuffer = Left$(sz, InStr(sz, vbNullChar) - 1)
  Else
    ' If sz had no null char, the Left$ function
    ' above would rtn a zero length string ("").
    GetStrFromBuffer = sz
  End If
End Function

' Returns True if sBuffer evaluates to a Unicode string
' Will sometimes evalutate an ANSII sBuffer as
' Unicode if terminated w/ chars other than null.

Public Function IsUnicodeStr(sBuffer As String) As Boolean
  Dim dwRtnFlags As Long
  dwRtnFlags = IS_TEXT_UNICODE_UNICODE_MASK
  IsUnicodeStr = IsTextUnicode(ByVal sBuffer, Len(sBuffer), dwRtnFlags)
'Debug.Print "out: " & dwRtnFlags
End Function

Public Sub DEFINE_GUID(name As GUID, l As Long, w1 As Integer, w2 As Integer, _
                                       b0 As Byte, b1 As Byte, b2 As Byte, b3 As Byte, _
                                       b4 As Byte, b5 As Byte, b6 As Byte, b7 As Byte)
  With name
    .Data1 = l
    .Data2 = w1
    .Data3 = w2
    .Data4(0) = b0
    .Data4(1) = b1
    .Data4(2) = b2
    .Data4(3) = b3
    .Data4(4) = b4
    .Data4(5) = b5
    .Data4(6) = b6
    .Data4(7) = b7
  End With
End Sub

Public Sub DEFINE_OLEGUID(name As GUID, l As Long, w1 As Integer, w2 As Integer)
  DEFINE_GUID name, l, w1, w2, &HC0, 0, 0, 0, 0, 0, 0, &H46
End Sub
