Attribute VB_Name = "mMain"
Option Explicit

' Brought to you by:
'   Brad Martinez
'   btmtz@msn.com
'   btmtz@aol.com
'   http://members.aol.com/btmtz/vb

' ================================================================
' A VB rendition of the SDK example "Using PIDLs and Display Names" found at:
'   http://www.microsoft.com/community/sdk/inetsdk/help/inet4536.htm

' The C code from the example has been directly ported to VB code providing nearly identical
' functionality. The original documentation and comments have been preserved as closely as
' possible from the example with exception of appropriate additions. It should be noted that
' references to a console application printing to a console window apply to this VB example
' printing to the Debug window. For more info on the shell's namespace, implementing the
' IShellFolder interface and pointers to item identifier lists (pidls), see the Shell Features and
' Extensions topic in the SDK.
' ================================================================
' From the SDK:

' This section presents an example illustrating how to retrieve the location of a special folder,
' walk an item identifier list, and use the IShellFolder interface to retrieve display names. The
' example is a console application that prints the display names of the folders a user would
' have to open to get to the Programs folder. To display them, the application would carry out
' these steps:
  
'  1.  Retrieve the PIDL (obtain a pointer to an item identifier list) for the Programs folder by
'       using the SHGetSpecialFolderLocation function.
'  2.  Bind to the desktop folder (retrieve the folder's IShellFolder interface) by using the
'       SHGetDesktopFolder function.
'  3.  Walk the item identifier list and process elements as follows: print the subfolder's display
'       name, bind to the subfolder, and release the parent folder's IShellFolder interface.
 
' Before carrying out any of the preceding steps, the application uses the SHGetMalloc
' function to retrieve a pointer to the shell's IMalloc interface, which it saves in the following
' global variable.

' Global pointer to the shell's IMalloc interface.
Public g_IMalloc As IMalloc
'

Sub main()
  main1
'  Do: main1: Loop   ' mem leak test
End Sub

' (The following example shows the application's main function.) This function carries out all
' of the steps described previously, although it calls the application-defined GetNextItemID
' and CopyItemID functions to walk the item identifier list and the application-defined
' PrintStrRet function to print the display names. Code for these application-defined
' functions is shown following the code for the main function.
' main - the application's entrypoint function

Public Function main1() As Integer
    Dim pMalloc As Long
    Dim pidlPrograms As Long
    Dim pFolder As Long
    Dim isfFolder As IShellFolder
    Dim IID_IShellFolder As GUID
    
    ' Fill IShellFolder Interface ID, {000214E6-0000-0000-C000-000000046}
    DEFINE_OLEGUID IID_IShellFolder, &H214E6, 0, 0
    
    ' Get the shell's allocator.
    If Not (SUCCEEDED(SHGetMalloc(pMalloc))) Then
        main1 = 1: Exit Function
    End If
    MoveMemory g_IMalloc, pMalloc, 4   ' Set g_IMalloc = pMalloc
    
    ' Get the PIDL for the Programs folder.
    If (SUCCEEDED(SHGetSpecialFolderLocation(0, _
            CSIDL_PROGRAMS, pidlPrograms))) Then
 
        ' Start with the desktop folder (we're bound to it)
        If (SUCCEEDED(SHGetDesktopFolder(pFolder))) Then
            MoveMemory isfFolder, pFolder, 4   ' Set isfFolder = pFolder
            Dim pidl As Long

            ' Process each item identifier in the list.
            pidl = pidlPrograms
            Do While pidl
                Dim sName As STRRET
                Dim pSubFolder As Long
                Dim pidlCopy As Long
 
                ' Copy the item identifier to a list by itself.
                pidlCopy = CopyItemID(pidl)
                If pidlCopy = 0 Then Exit Do
 
                ' Display the name of the subfolder.
                If (SUCCEEDED(isfFolder.GetDisplayNameOf(pidlCopy, _
                                                SHGDN_INFOLDER, sName))) Then
                    Call PrintStrRet(pidlCopy, sName)
                End If
                
                ' Bind to the subfolder.
                If Not (SUCCEEDED(isfFolder.BindToObject(pidlCopy, _
                                    0, IID_IShellFolder, pSubFolder))) Then
                    g_IMalloc.Free pidlCopy
                    Exit Do
                End If
                
                ' Free the copy of the item identifier.
                g_IMalloc.Free pidlCopy
 
                ' Release the parent folder and point to the subfolder.
                MoveMemory isfFolder, pSubFolder, 4   ' Set isfFolder = pSubFolder
                
                ' Get the next item ID
                pidl = GetNextItemID(pidl)   ' (was in the c for statement at top)
            Loop
 
            ' Release the last folder that was bound to.
            If Not (isfFolder Is Nothing) Then
                MoveMemory isfFolder, 0&, 4   ' Set isfFolder = Nothing
            End If
        End If
 
        ' Free the PIDL for the Programs folder.
        g_IMalloc.Free pidlPrograms
    End If
 
    ' Release the shell's allocator.
    MoveMemory g_IMalloc, 0&, 4   ' Set g_IMalloc = Nothing
    main1 = 0
 
End Function

' Following is the GetNextItemID function. Given a pointer to an element in an item identifier
' list, the function returns a pointer to the next element (or NULL if there are no more elements).
' The main function calls this function to walk the item identifier list for the Programs folder.
' GetNextItemID - points to the next element in an item identifier list.
' Returns a PIDL if successful or NULL if at the end of the list.
' pdil - previous element

Public Function GetNextItemID(pidl As Long) As Long
    Dim cb As Integer
    
    ' Get the size of the specified item identifier.
    MoveMemory cb, ByVal pidl, 2
 
    ' If the size is zero, it is the end of the list.
    If (cb = 0) Then
        GetNextItemID = 0: Exit Function
    End If
 
    ' Add cb to pidl (casting to increment by bytes).
    pidl = pidl + cb
 
    ' Return NULL if it is null-terminating or a pidl otherwise.
    MoveMemory cb, ByVal pidl, 2
    If cb Then GetNextItemID = pidl
 
End Function

' Following is the CopyItemID function. Given a pointer to an element in an item identifier
' list, the function allocates a new list containing only the specified element followed by a
' terminating zero. The main function uses this function to create single-element PIDLs,
' which it passes to IShellFolder member functions.
' CopyItemID - creates an item identifier list containing the first
'                     item identifier in the specified list.
' Returns a PIDL if successful or NULL if out of memory.

Public Function CopyItemID(pidl As Long) As Long
    Dim cb As Integer
    Dim iZero As Integer
    
    ' Get the size of the specified item identifier.
    MoveMemory cb, ByVal pidl, 2
 
    ' Allocate a new item identifier list.
    Dim pidlNew As Long
    pidlNew = g_IMalloc.Alloc(cb + 2)
    If (pidlNew = 0) Then
        CopyItemID = 0: Exit Function
    End If
 
    ' Copy the specified item identifier.
    MoveMemory ByVal pidlNew, ByVal pidl, cb
 
    ' Append a terminating zero.
    MoveMemory ByVal pidlNew + cb, iZero, 2
 
    CopyItemID = pidlNew

End Function
 
' The IShellFolder::GetDisplayNameOf member function returns a display name in a
' STRRET structure. The display name may be returned in one of three ways, which is
' specified by the uType member of the STRRET structure. The main function calls the
' following PrintStrRet function to print the display name.
' PrintStrRet - prints the contents of a STRRET structure.
' pidl - PIDL containing the display name if STRRET_OFFSET
' lpStr - address of the STRRET structure

Public Sub PrintStrRet(pidl As Long, lpStr As STRRET)
    Dim lpOleStr As Long
    Dim lpsz As Long
    Dim cch As Long
    Dim uOffset As Long
    Dim sOleStr
    
    Select Case (lpStr.uType)
 
        ' The 1st UINT (Long) of the array points to a Unicode
        ' str which must be freed (WinNT only).
        Case STRRET_WSTR:
            ' Get the pointer
            MoveMemory lpOleStr, lpStr.CStr(0), 4
            ' Get the strings len
            cch = WideCharToMultiByte(CP_OEMCP, 0, _
                                                      ByVal lpOleStr, -1, 0, _
                                                      0, vbNullString, 0)
            lpsz = g_IMalloc.Alloc(cch)
            If (lpsz <> 0) Then
                ' Map the Unicode str to ANSII using the ptrs
                Call WideCharToMultiByte(CP_OEMCP, 0, _
                                                       ByVal lpOleStr, -1, ByVal lpsz, _
                                                       cch, vbNullString, 0)
                Debug.Print GetStrFromPtr(lpsz, False)
                g_IMalloc.Free lpsz
            End If
 
        ' The 1st UINT (Long) of the array points to the location
        ' (uOffset bytes) to the ANSII str in the pidl
        Case STRRET_OFFSET:
            MoveMemory uOffset, lpStr.CStr(0), 4
            Debug.Print GetStrFromPtr(pidl + uOffset, False)
 
        ' The display name is returned in cStr
        Case STRRET_CSTR:
            Debug.Print GetStrFromPtr(VarPtr(lpStr.CStr(0)), False)
    
    End Select

End Sub
